import java.net.*;
import java.net.http.*;
import java.time.*;
import java.util.regex.*;


public class BookSearch{

  private static HttpRequest createRequest(String isbn){
    URI searchURI;
    try{
      searchURI = new URI("http",
                          "iss.ndl.go.jp",
                          "/api/sru",
                          "operation=searchRetrieve&query=isbn=" + isbn,
                          null);
    }
    catch(URISyntaxException e){
      throw new RuntimeException(e);
    }

    return HttpRequest.newBuilder()
                      .uri(searchURI)
                      .timeout(Duration.ofSeconds(90))
                      .build();
  }

  public static void main(String[] args) throws Exception{
    var client = HttpClient.newBuilder()
                           .version(HttpClient.Version.HTTP_1_1)
                           .followRedirects(HttpClient.Redirect.NORMAL)
                           .build();

    var pattern = Pattern.compile(
                      "^\\s*&lt;dc:title&gt;(.+?)&lt;/dc:title&gt;$");

    var book1 = client.sendAsync(createRequest(args[0]),
                                 HttpResponse.BodyHandlers.ofLines())
                      .thenApply(r -> r.body()
                                       .map(pattern::matcher)
                                       .filter(Matcher::matches)
                                       .findAny()
                                       .get()
                                       .group(1));

    var book2 = client.sendAsync(createRequest(args[1]),
                                 HttpResponse.BodyHandlers.ofLines())
                      .thenApply(r -> r.body()
                                       .map(pattern::matcher)
                                       .filter(Matcher::matches)
                                       .findAny()
                                       .get()
                                       .group(1));

    book1.thenAcceptBoth(book2,
                         (bk1, bk2) -> System.out.println(bk1 + ", " + bk2))
         .toCompletableFuture()
         .join();
  }

}
